<?php

namespace Drupal\diba_starterkit\Services;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Config\CachedStorage;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Extension\ModuleInstallerInterface;
use Drupal\Component\Serialization\Yaml;

/**
 * Implements StarterkitManager service.
 */
class StarterkitManager {

  /**
   * {@inheritdoc}
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   *
   * @var \Drupal\Core\Config\CachedStorage
   */
  protected $configStorage;

  /**
   * {@inheritdoc}
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * {@inheritdoc}
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleInstaller;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    ConfigFactory $configFactory,
    CachedStorage $configStorage,
    ModuleHandlerInterface $moduleHandler,
    ModuleInstallerInterface $moduleInstaller
  ) {
    $this->configFactory   = $configFactory;
    $this->configStorage   = $configStorage;
    $this->moduleHandler   = $moduleHandler;
    $this->moduleInstaller = $moduleInstaller;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.storage'),
      $container->get('module_handler'),
      $container->get('module_installer')
    );
  }

  /**
   * Get Diba starterkit defaults config files.
   */
  public function getConfigFiles($kit = 'standard') {
    $module_path = $this->moduleHandler->getModule('diba_starterkit')->getPath();
    $path = drupal_realpath($module_path . '/config/kits/' . $kit);

    return file_scan_directory($path, '/.yml$/', ['key' => 'name']);
  }

  /**
   * Apply Diba starterkit configs.
   *
   * @return array
   *   Lists of settings imported and modules installed/uninstalled.
   */
  public function apply($kit = 'standard') {
    $imported = $installed = $uninstalled = [];

    $modules = $this->getModules($kit);
    // Modules to install.
    foreach ($modules['uninstall'] as $module) {
      if ($this->moduleHandler->moduleExists($module)) {
        $this->moduleInstaller->uninstall([$module]);
        $uninstalled[] = $module;
      }
    }

    // Modules to uninstall.
    foreach ($modules['install'] as $module) {
      if (!$this->moduleHandler->moduleExists($module)) {
        $this->moduleInstaller->install([$module]);
        $installed[] = $module;
      }
    }

    // Apply configuration files.
    $configs = $this->getConfigFiles($kit);
    // Clear config cache to get real configs.
    $this->configFactory->reset();
    $currentConfigs = $this->configFactory->listAll();

    foreach ($configs as $key => $file) {
      $configuration = Yaml::decode(file_get_contents($file->uri));

      if (in_array($key, $currentConfigs)) {
        // If the config exists override.
        if ($config = $this->configFactory->getEditable($key)) {
          $config->setData($configuration)->save();
          $imported[] = $key;
        }
      }
      else {
        // If the config not exists create.
        $this->configStorage->write($key, $configuration);
      }
    }

    return [
      'installed'   => $installed,
      'uninstalled' => $uninstalled,
      'imported'    => $imported,
    ];
  }

  /**
   * Get modules to install/uninstall byt kit.
   *
   * @return array
   *   List of modules to install/uninstall.
   */
  private function getModules($kit) {
    // Basic settings kit.
    $kit_settings = [
      'install' => [
        'automated_cron',
        'locale',
        'syslog',
      ],
      'uninstall' => [
        'help',
        'tour',
        'shortcut',
        'quickedit',
        'color',
      ],
    ];

    // Invoke HOOK_starterkit_KIT_alter() to attach more settings.
    $this->moduleHandler->alter('starterkit_' . $kit, $kit_settings);

    return $kit_settings;
  }

}
